#!/usr/bin/env python3

import pytest
import pyranges as pr

def assert_equal_length_before_after(gr1, gr2):

    print("in test")
    l1 = len(gr1)
    l2 = len(gr2)
    c = pr.concat([gr1, gr2])

    if not gr1.stranded or not gr2.stranded:
        assert not c.stranded

    lc = len(c)
    assert l1 + l2 == lc

def test_concat_stranded_unstranded(f1, f2):

    assert_equal_length_before_after(f1, f2)

def test_concat_unstranded_unstranded(f1, f2):

    assert_equal_length_before_after(f1.unstrand(), f2.unstrand())

def test_concat_stranded_unstranded(f1, f2):
    assert_equal_length_before_after(f1, f2.unstrand())

def test_concat_unstranded_stranded(f1, f2):
    assert_equal_length_before_after(f1.unstrand(), f2)




def get_chromosomes_from_dict(dfs):

    keys = list(dfs.keys())
    if isinstance(keys[0], tuple):
        chromosomes = [k[0] for k in keys]
    else:
        chromosomes = keys

    return chromosomes



def get_strands_from_dict(dfs):

    keys = list(dfs.keys())
    if isinstance(keys[0], tuple):
        strands = [k[1] for k in keys]
    else:
        strands = keys

    return strands



def get_key_from_df(df):

    chromosome = df.Chromosome.head(1).iloc[0]
    if "Strand" in df:
        strand = df.Strand.head(1).iloc[0]
        return chromosome, strand

    return chromosome


def single_value_key(df):

    if "Strand" in df:
        return len(df[["Chromosome", "Strand"]].drop_duplicates(["Chromosome", "Strand"])) == 1
    else:
        return len(df.Chromosome.drop_duplicates()) == 1






import pandas as pd


def assert_df_equal(df1, df2):

    # df1.loc[:, "Start"] = df1.Start.astype(np.int64)
    # df2.loc[:, "Start"] = df1.Start.astype(np.int64)
    # df1.loc[:, "End"] = df1.End.astype(np.int64)
    # df2.loc[:, "End"] = df1.End.astype(np.int64)

    pd.options.mode.chained_assignment = None
    if "Strand" in df1 and "Strand" in df2:
        sort_on = "Chromosome Start End Strand".split()
        df1.Strand = df1.Strand.astype("object")
        df2.Strand = df2.Strand.astype("object")
    else:
        sort_on = "Chromosome Start End".split()

    if "Strand_b" in df1:
        sort_on += "Start_b End_b Strand_b".split()
        df1.Strand_b = df1.Strand_b.astype("object")
        df2.Strand_b = df2.Strand_b.astype("object")
    elif "Start_b" in df2:
        sort_on += "Start_b End_b".split()

    df1 = df1.sort_values(sort_on)
    df2 = df2.sort_values(sort_on)

    df1 = df1.reset_index(drop=True)
    df2 = df2.reset_index(drop=True)

    df1.Chromosome = df1.Chromosome.astype("object")
    df2.Chromosome = df2.Chromosome.astype("object")

    print("Actual")
    print(df1.to_csv(sep=" "))
    print("Expected")
    print(df2.to_csv(sep=" "))

    print("Actual dtypes")
    print(df1.dtypes)
    print("Expected dtypes")
    print(df2.dtypes)
    # print("dtypes Strand\n", "1",  df1.Strand.dtype, "2", df2.Strand.dtype)
    # print("dtypes Strand\n", df1.Strand.dtype == df2.Strand.dtype)
    # print("dtypes equal\n", df1.dtypes == df2.dtypes)

    print("Actual index")
    print(df1.index)
    print("Expected index")
    print(df2.index)
    print("index equal", df1.index == df2.index)

    pd.testing.assert_frame_equal(df1, df2)

    pd.options.mode.chained_assignment = "warn"
